SystemOrganization addCategory: #'EjerciciosPoo-TusLibros'!

Object subclass: #Cart
	instanceVariableNames: 'catalog items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

Cart class
	instanceVariableNames: 'lastCartId'!

Cart class
	instanceVariableNames: 'lastCartId'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/13/2012 21:25'!
newWithCatalog: aCatalog
	
	^self new initializeWithCatalog: aCatalog! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RaulBenitti 6/17/2012 11:31'!
= aCart
	
	^(items = aCart items) and: (catalog = aCart catalog)! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RaulBenitti 6/17/2012 11:33'!
add: anItem
	
	self add: anItem withOccurrences: 1.
	^anItem ! !

!Cart methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 09:24'!
add: anItem withOccurrences: numberOfOccurrences
	
	(numberOfOccurrences > 0)
		ifTrue: [self verifyIfAnItem: anItem existsInCatalogAndDo: [items add: anItem withOccurrences: numberOfOccurrences]]
		ifFalse: [Error signal: 'Invalid number of occurrences. It cannot be added.']! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RaulBenitti 6/17/2012 11:35'!
amountOf: anItem 
	
	^items occurrencesOf: anItem ! !

!Cart methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 15:06'!
catalog 
	
	^catalog deepCopy.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RaulBenitti 6/17/2012 11:16'!
hash
	
	^((items hash) + (catalog hash)) hash ! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RaulBenitti 6/17/2012 11:46'!
initializeWithCatalog: aCatalog
	"The only requirement on aCatalog is that it must know the message includes"	
	
	catalog := aCatalog.
	items := OrderedCollection new.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/13/2012 21:31'!
isEmpty 
	
	^items isEmpty ! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RaulBenitti 6/17/2012 12:19'!
items
	" Returns an Ordered Collection with the items in the cart"
	|anOrderCollectionOfItems|
	
	anOrderCollectionOfItems := OrderedCollection new. 
	items do:[ :anItem | anOrderCollectionOfItems add: anItem ].
	
	^anOrderCollectionOfItems! !

!Cart methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 20:14'!
reset
	
	items := OrderedCollection new.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/13/2012 21:43'!
size
	
	^items size! !

!Cart methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 11:35'!
verifyIfAnItem: anItem existsInCatalogAndDo: aBlock
	
	(catalog includesKey: anItem) 
		ifTrue: aBlock
		ifFalse: [Error signal: 'Item is not present in the catalog. It cannot be added.']! !

!Cart methodsFor: 'as yet unclassified' stamp: 'benitti 6/18/2012 18:32'!
withItemsDo: aBlock
	
	items do: aBlock! !

Object subclass: #Cashier
	instanceVariableNames: 'salesBook merchantProcessorInterface'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 11:23'!
newWithSalesBook: aSalesBook andWithMerchantProcessorInterface: aMerchantProcessorInterface
	
	^Cashier new initializeWithSalesBook: aSalesBook andWithMerchantProcessorInterface: aMerchantProcessorInterface! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 13:09'!
addSaleTo: aClientName withCart: aCart withCatalog: aCatalog

	| aDetail totalAmount |
	
	aDetail := Set new.
	totalAmount := 0.
	aCart withItemsDo: [:anItem | 
							totalAmount := totalAmount + (aCatalog at: anItem).
							aDetail add: (self createDetailForAnItem: anItem withQuantity: (aCart amountOf: anItem) withPrice: (aCatalog at: anItem)).
						].
	^salesBook add: (Sale newWithClient: aClientName withDetail: aDetail withTotalAmount: totalAmount)! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 20:13'!
chargeACreditCard: aCreditCard fromACart: aCart withCatalog: aCatalog
	
	| totalAmount aSale |
	
	self verifyCartNotEmpty: aCart andDo: [
			self verifyCreditCardNotExpired: aCreditCard andDo: [
				totalAmount := 0.
				aCart withItemsDo: [:anItem | totalAmount := totalAmount + (aCatalog at: anItem)].
				(merchantProcessorInterface debitFrom: aCreditCard amount: totalAmount)
					ifTrue: [aSale := self addSaleTo: aCreditCard ownerName withCart: aCart withCatalog: aCatalog.
							aCart reset.
							^ aSale.]
					ifFalse: [Error signal: 'Could not charge from creditCard']
				] 	
			]! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 13:08'!
createDetailForAnItem: anItem withQuantity: aQuantity withPrice: aPrice

	^(OrderedCollection new) 
			add: anItem; yourself; 
			add: aQuantity; yourself; 
			add: aPrice; 
			yourself! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 11:24'!
initializeWithSalesBook: aSalesBook andWithMerchantProcessorInterface: aMerchantProcessorInterface
	
	salesBook := aSalesBook.
	merchantProcessorInterface := aMerchantProcessorInterface.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 15:08'!
salesBook
	
	^salesBook deepCopy.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 11:09'!
verifyCartNotEmpty: aCart andDo: aBlock
	
	(aCart isEmpty)
		ifTrue: [Error signal: 'An empty cart cannot be charged.']
		ifFalse: aBlock! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 11:11'!
verifyCreditCardNotExpired: aCreditCard andDo: aBlock
	
	(aCreditCard isExpiredOn: (MonthOfYear onMonthNumber: (Date current monthIndex) of: (Date current year)))
		ifTrue: [self error: 'Cannot debit from an expired credit card.']
		ifFalse: aBlock! !

Object subclass: #Rest
	instanceVariableNames: 'activeCarts cashier merchantProcessorInterface catalog users salesBook nextCartId nextTransactionId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Rest class methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 15:36'!
newWithMerchantProcessorInterface: aMerchantProcessorInterface andWithCatalog: aCatalog andWithUsers: aListOfUsers
	
	^Rest new initializeWithMerchantProcessorInterface: aMerchantProcessorInterface andWithCatalog: aCatalog andWithUsers: aListOfUsers.! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 15:48'!
activeCarts
	
	| carts |
	
	carts := Set new.
	activeCarts keysDo: [:key | carts add: key].
	^carts.! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 17:50'!
addToCart: aCartId aBook: aBook quantity: aQuantity
	
	self verifyCartIsActive: aCartId 
		andDo: [(activeCarts at: aCartId) add: aBook withOccurrences: aQuantity]
		orRaiseExceptionWithMessage: 'Cannot add to an invalid cart.'! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 18:51'!
checkoutCartId: aCartId withCreditCardNumber: aCreditCardNumber withExpirationDate: anExpirationDate withOwnerName: aName

	| aCreditCard aTransactionId|
	
	self verifyCartIsActive: aCartId 
		andDo: [
				aCreditCard := CreditCard ownedBy: aName identifiedBy: aCreditCardNumber expiresOn: anExpirationDate.
	
				cashier chargeACreditCard: aCreditCard fromACart: (self getCart: aCartId) withCatalog: catalog.
				^self nextTransactionId.
			]
		orRaiseExceptionWithMessage: 'Cannot checkout an invalid cart.'
	
	
	! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 17:23'!
createCartForUser: anUser withPassword: aPassword
	
	| cartId |
	
	(users includesKey: anUser)
		ifTrue: [
			((users at: anUser) = aPassword)
				ifTrue: [
						cartId := nextCartId.
						self incrementCartId.
						activeCarts at: cartId put: (Cart newWithCatalog: catalog).
						^cartId.
					]
				ifFalse: [Error signal: 'Cannot create a cart for invalid Password.']
			]
		ifFalse: [Error signal: 'Cannot create a cart for invalid User.']! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 16:35'!
getCart: aCartId
	
	^activeCarts at: aCartId.! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 17:22'!
incrementCartId 
	
	nextCartId := nextCartId + 1! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 18:49'!
initializeWithMerchantProcessorInterface: aMerchantProcessorInterface andWithCatalog: aCatalog andWithUsers: aListOfUsers
	
	nextCartId := 1.
	nextTransactionId := 1.
	activeCarts := Dictionary new.
	salesBook := OrderedCollection new.
	cashier := Cashier newWithSalesBook: salesBook andWithMerchantProcessorInterface: aMerchantProcessorInterface.
	merchantProcessorInterface := aMerchantProcessorInterface.
	catalog := aCatalog.
	users := aListOfUsers.! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 17:50'!
listCart: aCartId
	
	| items |
	
	items := Bag new.
	self verifyCartIsActive: aCartId 
		andDo: [
				(activeCarts at: aCartId) withItemsDo: [:anItem | items add: anItem].
				^items.]
		orRaiseExceptionWithMessage: 'Cannot list an invalid cart.'! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 20:04'!
listPurchasesForAUser: anUser withPassword: aPassword
	
	| purchases totalAmount |
	
	purchases := Dictionary new.
	totalAmount := 0.
	
	(users includesKey: anUser)
		ifTrue: [
			((users at: anUser) = aPassword)
				ifTrue: [
						salesBook 
							select: [ :aSale | aSale clientName = anUser ] 
							thenDo:[ :aSale |
									totalAmount := totalAmount + aSale totalAmount.
									aSale detail do: [ :anItem | 
													(purchases includesKey: (anItem at: 1)) 
														ifTrue: [purchases at: (anItem at: 1) put: (purchases at: (anItem at: 1)) + (anItem at: 2) ] 
														ifFalse: [purchases at: (anItem at: 1) put: (anItem at: 2) ]
														]
									].
						purchases at: 'totalAmount' put: totalAmount.
						^purchases.
					]
				ifFalse: [Error signal: 'Cannot list puchases of an user with an invalid password.']
			]
		ifFalse: [Error signal: 'Cannot list puchases of an invalid user.']! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 17:24'!
nextCartId
	
	^nextCartId! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 18:48'!
nextTransactionId 
	
	| transactionId |
 
	transactionId := nextTransactionId.	
	nextTransactionId := nextTransactionId + 1.
	^transactionId.	! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 18:57'!
salesBook 
	
	^salesBook deepCopy.! !

!Rest methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 17:49'!
verifyCartIsActive: aCartId andDo: aBlock orRaiseExceptionWithMessage: aMessage

	(activeCarts includesKey: aCartId)
		ifTrue: aBlock
		ifFalse: [Error signal: aMessage]! !

Object subclass: #Sale
	instanceVariableNames: 'clientName detail totalAmount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Sale class methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 10:57'!
newWithClient: aClientName withDetail: aDetail withTotalAmount: totalAmount

	^Sale new initializeWithClient: aClientName withDetail: aDetail withTotalAmount: totalAmount! !

!Sale methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 13:19'!
= aSale

	^((totalAmount = aSale totalAmount) and: [clientName = aSale clientName]) and: [detail = aSale detail]! !

!Sale methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 13:17'!
clientName  

	^clientName! !

!Sale methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 15:09'!
detail  

	^detail deepCopy! !

!Sale methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 19:07'!
hash

	^((totalAmount hash) * (clientName hash) * (detail hash)) hash! !

!Sale methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 10:57'!
initializeWithClient: aClientName withDetail: aDetail withTotalAmount: aTotalAmount

	clientName := aClientName.
	detail := aDetail.
	totalAmount := aTotalAmount.! !

!Sale methodsFor: 'as yet unclassified' stamp: 'IgalFrid 6/24/2012 11:40'!
totalAmount 

	^totalAmount! !

TestCase subclass: #TusLibrosTest
	instanceVariableNames: 'numberOfMPCalls'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!TusLibrosTest methodsFor: 'books' stamp: 'IgalFrid 6/24/2012 09:26'!
BookNotInCatalog 
	
	^'anItemNotInCatalog'.! !

!TusLibrosTest methodsFor: 'sales' stamp: 'IgalFrid 6/24/2012 19:04'!
aSaleOfOneBook1
	
	^Sale 
		newWithClient: self validCreditCard ownerName 
		withDetail: ((Set new) add: ((OrderedCollection new) add:(self book1); yourself; add: 1; yourself; add:((self catalog) at: self book1); yourself); yourself) 
		withTotalAmount: ((self catalog) at: self book1).! !

!TusLibrosTest methodsFor: 'sales' stamp: 'IgalFrid 6/24/2012 19:08'!
aSaleOfThreeBook2
	
	^Sale 
		newWithClient: self validCreditCard ownerName 
		withDetail: ((Set new) add: ((OrderedCollection new) add:(self book2); yourself; add: 3; yourself; add:((self catalog) at: self book2); yourself); yourself) 
		withTotalAmount: ((self catalog) at: self book2) * 3.! !

!TusLibrosTest methodsFor: 'cashiers' stamp: 'IgalFrid 6/24/2012 11:39'!
aSalesBook
	
	^ Set new! !

!TusLibrosTest methodsFor: 'books' stamp: 'IgalFrid 6/24/2012 12:03'!
book1 

	^'Book10'! !

!TusLibrosTest methodsFor: 'books' stamp: 'IgalFrid 6/24/2012 12:03'!
book2 

	^'Book11'! !

!TusLibrosTest methodsFor: 'carts' stamp: 'IgalFrid 6/24/2012 12:03'!
cartWithBook1 
	
	| cart |
	
	cart := self emptyCart.
	cart add: self book1.
	^cart! !

!TusLibrosTest methodsFor: 'carts' stamp: 'IgalFrid 6/24/2012 12:03'!
cartWithOneBookFromEditorial1 
	
	| cart |
	
	cart := Cart newWithCatalog:  self catalog.
	cart add: (self book1).
	
	^cart! !

!TusLibrosTest methodsFor: 'carts' stamp: 'IgalFrid 6/24/2012 12:03'!
cartWithTwoBooksFromEditorial1 
	
	| cart |
	
	cart := Cart newWithCatalog:  self catalog.
	cart add: (self book1) withOccurrences: 4.
	cart add: (self book2) withOccurrences: 3.
	
	^cart! !

!TusLibrosTest methodsFor: 'cashiers' stamp: 'IgalFrid 6/24/2012 11:39'!
cashier
	^ Cashier newWithSalesBook: self aSalesBook andWithMerchantProcessorInterface: self merchantProcessorInterface! !

!TusLibrosTest methodsFor: 'catalogs' stamp: 'IgalFrid 6/24/2012 11:30'!
catalog
	| catalog |
	
	catalog:= Dictionary new.
	catalog 
		at: 'Book10' put: 10;
		at: 'Book11' put: 11; 
		at: 'Book12' put: 12;
		at: 'Book20' put: 20;
		at: 'Book21' put: 21; 
		at: 'Book22' put: 22;
		at: 'Book30' put: 30;
		at: 'Book31' put: 31;
		at: 'Book32' put: 32.
	^catalog! !

!TusLibrosTest methodsFor: 'merchantProcessor' stamp: 'IgalFrid 6/24/2012 13:43'!
debitFrom: aCreditCard amount: totalAmount

	numberOfMPCalls := numberOfMPCalls + 1.
	^(aCreditCard number = 2222) not
! !

!TusLibrosTest methodsFor: 'carts' stamp: 'IgalFrid 6/24/2012 09:48'!
emptyCart 
	
	^Cart newWithCatalog: self catalog.! !

!TusLibrosTest methodsFor: 'creditcards' stamp: 'benitti 6/18/2012 18:24'!
expiredCreditCard 
	
	^CreditCard ownedBy: 'Owner' identifiedBy: 1111 expiresOn: (MonthOfYear januaryOf: (Date current year - 1))
 
 ! !

!TusLibrosTest methodsFor: 'creditcards' stamp: 'IgalFrid 6/24/2012 13:42'!
invalidCreditCard 
	
	^CreditCard ownedBy: 'Owner' identifiedBy: 2222 expiresOn: (MonthOfYear januaryOf: (Date current year + 1))
 ! !

!TusLibrosTest methodsFor: 'users' stamp: 'IgalFrid 6/24/2012 15:38'!
invalidUser 
	
	^'invalidUser'! !

!TusLibrosTest methodsFor: 'users' stamp: 'IgalFrid 6/24/2012 15:35'!
listOfUsers 
	
	| users |
	
	users := Dictionary new.
	users 
		at: 'user1' put: 'password1';
		at: 'user2' put: 'password2'.
		
	^users! !

!TusLibrosTest methodsFor: 'merchantProcessor' stamp: 'IgalFrid 6/24/2012 10:25'!
merchantProcessorInterface

	numberOfMPCalls := 0. 
	^self! !

!TusLibrosTest methodsFor: 'carts' stamp: 'IgalFrid 6/24/2012 12:03'!
nonEmptyCart 
	
	| cart |
	cart := self emptyCart.
	cart add: self book1 withOccurrences: 1.
	cart add: self book2 withOccurrences: 2.
	 
	^cart.! !

!TusLibrosTest methodsFor: 'merchantProcessor' stamp: 'IgalFrid 6/24/2012 10:26'!
numberOfCalls

	^numberOfMPCalls! !

!TusLibrosTest methodsFor: 'users' stamp: 'IgalFrid 6/24/2012 15:38'!
password1 
	
	^'password1'! !

!TusLibrosTest methodsFor: 'rests' stamp: 'IgalFrid 6/24/2012 15:36'!
rest
	
	^Rest newWithMerchantProcessorInterface: self merchantProcessorInterface andWithCatalog: self catalog andWithUsers: self listOfUsers.! !

!TusLibrosTest methodsFor: 'rests' stamp: 'IgalFrid 6/24/2012 17:29'!
restWithOneEmptyCart
	
	| rest |
	rest := Rest newWithMerchantProcessorInterface: self merchantProcessorInterface andWithCatalog: self catalog andWithUsers: self listOfUsers.
	rest createCartForUser: self user1 withPassword: self password1.
	
	^rest.! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/13/2012 21:24'!
test01NewCartShouldBeEmpty

	self assert: self emptyCart isEmpty! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 12:03'!
test02CanAddABookFromTheCatalog

	| cart |
	
	cart := self emptyCart.
	cart add: self book1 withOccurrences:1.
	self deny: cart isEmpty! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 09:29'!
test03CannotAddABookNotInTheCatalog
	
	| anEmptyCart |
	
	anEmptyCart := self emptyCart.
	self should: [anEmptyCart add: self BookNotInCatalog] 
		raise: Error
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Item is not present in the catalog. It cannot be added.'.
							self assert: anEmptyCart isEmpty 
							]! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 12:03'!
test04CanAddBooksInTheCatalogWithMoreThanOneOccurrence

	| cart |
	
	cart := self emptyCart.
	cart add: self book1 withOccurrences: 2.
	self assert: cart size = 2 ! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 09:31'!
test05CannotAddBooksNotInTheCatalogWithMoreThanOneOccurrence

	| cart |
	
	cart := self emptyCart.
	
	self should: [cart add: self BookNotInCatalog  withOccurrences: 2.] 
		raise: Error
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Item is not present in the catalog. It cannot be added.'.
							self assert: cart size = 0.
						].
	
! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 12:03'!
test06CannotAddBooksInTheCatalogWithLessThanOneOccurrence
	
	| aCart |
	
	aCart := self emptyCart.
	self should: [ aCart add: self book1 withOccurrences: 0] 
		raise: Error
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Invalid number of occurrences. It cannot be added.'.
							self assert: aCart isEmpty 
						].
	self should: [ aCart add: self book1 withOccurrences: -2]
		raise: Error
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Invalid number of occurrences. It cannot be added.'.
							self assert: aCart isEmpty 
						].! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 09:32'!
test07CannotAddBooksNotInCatalogWithLessThanOneOccurrence
	
	| aCart |
	
	aCart := self emptyCart.
	self should: [ aCart add: self BookNotInCatalog withOccurrences: 0] 
		raise: Error
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Invalid number of occurrences. It cannot be added.'.
							self assert: aCart isEmpty 
						].
	self should: [ aCart add: self BookNotInCatalog withOccurrences: -2]
		raise: Error
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Invalid number of occurrences. It cannot be added.'.
							self assert: aCart isEmpty 
						].! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 12:03'!
test08aCartHasACorrectAmountOfBooks

	| cart |
	
	cart := self emptyCart.
	cart add: self book1 withOccurrences: 4.
	cart add: self book1 withOccurrences: 3.
	
	self assert: (cart amountOf: self book1) = 7. 
 
	cart add: self book1 withOccurrences: 3.
	
	self assert: (cart size) = 10. 
 ! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 09:36'!
test08bOccurrencesOfBooksNotInCartIsZero

	self assert: (self emptyCart amountOf: 'Book10') = 0 
! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 11:36'!
test09CartCorrectlyListTheAddedItems

	| aCart  |
	
	aCart := self emptyCart.
	
	self catalog keys do:[ :anItem |
			aCart add: anItem].
	aCart items do:[ :anItem |
			self assert: ((self catalog ) includesKey: anItem )] 
	! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 12:03'!
test10TheCorrectAmountOfBooksIsAddedToTheCart

	| cart |
	
	cart := self emptyCart.
	cart add: self book1 withOccurrences: 4.
	cart add: self book1 withOccurrences: 3.
	cart add: self book2 withOccurrences: 3.
	
	self assert: (cart amountOf:   self book1) = 7.  
	self assert: (cart amountOf:  self book2) = 3.
	self assert: (cart size) = 10. 
 ! !

!TusLibrosTest methodsFor: '1 - Test cart' stamp: 'IgalFrid 6/24/2012 09:34'!
test11TwoNewCartsShouldBeEquals

	self assert: (self emptyCart = self emptyCart)! !

!TusLibrosTest methodsFor: '2 - Test cashier' stamp: 'IgalFrid 6/24/2012 14:48'!
test12ACashierCannotChargeAnEmpyCart
	| aCashier aBookSales aCart numberOfCalls |
	aCashier := self cashier.
	aBookSales := aCashier salesBook.
	aCart := self emptyCart.
	numberOfCalls := self merchantProcessorInterface numberOfCalls.
	self
		should: [ aCashier chargeACreditCard: self validCreditCard fromACart: aCart withCatalog: aCart catalog ]
		raise: Error
		withExceptionDo: [ :anException | 
			self assert: anException messageText = 'An empty cart cannot be charged.'.
			self assert: aBookSales = aCashier salesBook.
			self assert: self merchantProcessorInterface numberOfCalls = numberOfCalls
			]! !

!TusLibrosTest methodsFor: '2 - Test cashier' stamp: 'IgalFrid 6/24/2012 14:48'!
test13ACashierCannotChargeACartWithAnExpiredCreditCard
	| aCashier aBookSales aCart numberOfCalls |
	aCashier := self cashier.
	aBookSales := aCashier salesBook.
	aCart := self cartWithBook1.
	numberOfCalls := self merchantProcessorInterface numberOfCalls.
	self
		should: [ aCashier chargeACreditCard: self expiredCreditCard fromACart: aCart withCatalog: aCart catalog ]
		raise: Error
		withExceptionDo: [ :anException | 
			self assert: anException messageText = 'Cannot debit from an expired credit card.'.
			self assert: aBookSales = aCashier salesBook.
			self assert: self merchantProcessorInterface numberOfCalls = numberOfCalls
			]
		
	
	! !

!TusLibrosTest methodsFor: '2 - Test cashier' stamp: 'IgalFrid 6/24/2012 14:48'!
test14AmountOfChargeACartWithMoreThanOneItemIsCorrect
	
	| aCart totalAmount |
	
	totalAmount := 0.
	aCart := self cartWithTwoBooksFromEditorial1.
	aCart withItemsDo: [:anItem | 
							totalAmount := totalAmount + (aCart catalog at: anItem).
						].
	self assert: ((self cashier chargeACreditCard: self validCreditCard fromACart: aCart withCatalog: aCart catalog) totalAmount) = totalAmount! !

!TusLibrosTest methodsFor: '2 - Test cashier' stamp: 'IgalFrid 6/24/2012 14:48'!
test15AmountOfChargeACartWithOneItemIsCorrect

	| aCart |
	
	aCart := self cartWithOneBookFromEditorial1.
	self assert: ((self cashier chargeACreditCard: self validCreditCard fromACart: aCart withCatalog: aCart catalog) totalAmount) = 10! !

!TusLibrosTest methodsFor: '2 - Test cashier' stamp: 'IgalFrid 6/24/2012 19:04'!
test16ACashierCanSuccesfullyChargeAValidCreditCardWithAnNonEmpyCart
	
	| aCart aCashier numberOfCalls aMerchantProcessorInterface aSalesBook aSale  anotherSale |
	
	aMerchantProcessorInterface := self merchantProcessorInterface.
	aCashier := self cashier.
	aCart := self cartWithOneBookFromEditorial1 .
	numberOfCalls := self merchantProcessorInterface numberOfCalls.
	aSalesBook := aCashier salesBook.
	aSale := self aSaleOfOneBook1.
	
	aCashier chargeACreditCard: self validCreditCard fromACart: aCart withCatalog: aCart catalog.
	
	self assert: aMerchantProcessorInterface numberOfCalls = (numberOfCalls + 1).
	
	aSalesBook add: aSale.
	self assert: aCashier salesBook = aSalesBook.! !

!TusLibrosTest methodsFor: '2 - Test cashier' stamp: 'IgalFrid 6/24/2012 18:23'!
test17CannotChargeIfMerchantProcessorRejectedTheOperation
	
	| aCart aCashier numberOfCalls aMerchantProcessorInterface aSalesBook |
	
	aMerchantProcessorInterface := self merchantProcessorInterface.
	aCashier := self cashier.
	aCart := self cartWithOneBookFromEditorial1 .
	numberOfCalls := self merchantProcessorInterface numberOfCalls.
	aSalesBook := aCashier salesBook.
	
	self should: [ aCashier chargeACreditCard: self invalidCreditCard fromACart: aCart withCatalog: aCart catalog. ]
	raise: Error
	withExceptionDo: [ :anException | 
			self assert: anException messageText = 'Could not charge from creditCard'.
			self assert: aSalesBook = aCashier salesBook.
			self assert: aMerchantProcessorInterface numberOfCalls = (numberOfCalls + 1).
			]
	
! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 15:04'!
test18CreateCartWithInvalidUserShouldReturnAnError
	
	| aRest carts |
	
	aRest := self rest.
	carts := aRest activeCarts.
	self should: [
			aRest createCartForUser: self invalidUser withPassword: 'xxx'
			]
		raise: Error
		withExceptionDo: [ :anException | 
			self assert: anException messageText = 'Cannot create a cart for invalid User.'.
			self assert: carts = aRest activeCarts.
			]
	
	! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 15:38'!
test19CreateCartWithInvalidPasswordShouldReturnAnError
	
	| aRest carts |
	
	aRest := self rest.
	carts := aRest activeCarts.
	self should: [
			aRest createCartForUser: self user1 withPassword: 'xxx'
			]
		raise: Error
		withExceptionDo: [ :anException | 
			self assert: anException messageText = 'Cannot create a cart for invalid Password.'.
			self assert: carts = aRest activeCarts.
			]
	
	! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 15:46'!
test20CreateCartWithValidUserAndPasswordShouldReturnTheCartId
	
	| aRest carts cartId |
	
	aRest := self rest.
	carts := aRest activeCarts.
	
	cartId := aRest createCartForUser: self user1 withPassword: self password1.
	
	carts add: cartId.	
	self assert: carts = aRest activeCarts.
	
	! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 17:29'!
test21CannotAddToAnInvalidCart
	
	|aRest|
	 
	aRest := self restWithOneEmptyCart.
	self testRest: aRest methodAddToCart: 0 aBook: self book1 aQuantity: 1 withExceptionMessage: 'Cannot add to an invalid cart.'.! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 17:29'!
test22CannotAddToAnInvalidCart
	
	|aRest|
	 
	aRest := self restWithOneEmptyCart.
	self testRest: aRest methodAddToCart: aRest activeCarts someElement aBook: self BookNotInCatalog aQuantity: 1 withExceptionMessage: 'Item is not present in the catalog. It cannot be added.'.! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 17:29'!
test23CannotAddAnInvalidQuantityToACart
	
	|aRest|
	 
	aRest := self restWithOneEmptyCart.
	self testRest: aRest methodAddToCart: aRest activeCarts someElement aBook: self book1 aQuantity: 0 withExceptionMessage: 'Invalid number of occurrences. It cannot be added.'.! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 17:29'!
test24CanAddABookToACart
	
	| someCartId aRest previousCarts previousActiveCarts |
	
	aRest := self restWithOneEmptyCart.
	previousActiveCarts := aRest activeCarts. 
	previousCarts := Dictionary new.
	aRest activeCarts do: [:aCartId | previousCarts at: aCartId put: (aRest getCart: aCartId)].
	someCartId := aRest activeCarts someElement.
	
	aRest addToCart: someCartId aBook: self book1 quantity: 2.
	self assert: aRest activeCarts = previousActiveCarts.
	aRest activeCarts do: [:aCartId | 
		(aCartId = someCartId)
			ifTrue: [self assert: (aRest getCart: aCartId) = ((previousCarts at: aCartId) add: self book1 withOccurrences: 2)]
			ifFalse: [self assert: (aRest getCart: aCartId) = (previousCarts at: aCartId)]
		]! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 17:29'!
test25CannotListAnInvalidCart
	
	|aRest|
	 
	aRest := self restWithOneEmptyCart.
	self should: [aRest listCart: 0]
		raise: Error 
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Cannot list an invalid cart.']! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 17:40'!
test26CanListAValidCart
	
	| aCartId aRest items aBag |
	
	aBag := Bag new.
	items := Bag new. 
	items add: self book1 withOccurrences: 2.
	items add: self book2 withOccurrences: 3.
	
	aRest := self restWithOneEmptyCart.
	
	aCartId := aRest activeCarts someElement.
	aRest addToCart: aCartId aBook: self book1 quantity: 2.
	aRest addToCart: aCartId aBook: self book2 quantity: 3.
	
	(aRest listCart: aCartId) do:[:aBook |  aBag add: aBook].
	self assert: aBag = items.! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 18:51'!
test27CannotCheckOutAnInvalidCart
	
	| aRest aCreditCard |
	 
	aRest := self restWithOneEmptyCart.
	self should: [aRest checkoutCartId: 0 withCreditCardNumber: 1111 withExpirationDate: (MonthOfYear januaryOf: (Date current year + 1)) withOwnerName: 'xxx']
		raise: Error 
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Cannot checkout an invalid cart.']! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 19:00'!
test28CannotCheckoutWithAnExpiredCreditCard
	
	| aRest aCreditCard aMerchantProcessorInterface aSalesBook numberOfCalls someCartId |
	
	aRest := self restWithOneEmptyCart.
	numberOfCalls := self merchantProcessorInterface numberOfCalls.
	aSalesBook := aRest salesBook.
	aMerchantProcessorInterface := self merchantProcessorInterface.
	aCreditCard := self expiredCreditCard.
	
	someCartId := aRest activeCarts someElement.
	aRest addToCart: someCartId aBook: self book1 quantity: 2.
	self should: [aRest checkoutCartId: someCartId withCreditCardNumber: aCreditCard number withExpirationDate: aCreditCard expirationDate withOwnerName: aCreditCard ownerName.]
		raise: Error 
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Cannot debit from an expired credit card.'.
							self assert: aSalesBook = aRest salesBook.
							self assert: aMerchantProcessorInterface numberOfCalls = numberOfCalls]! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 18:59'!
test29CannotCheckoutAnEmptyCart
	
	| aRest aCreditCard aMerchantProcessorInterface aSalesBook numberOfCalls someCartId |
	
	aRest := self restWithOneEmptyCart.
	numberOfCalls := self merchantProcessorInterface numberOfCalls.
	aSalesBook := aRest salesBook.
	aMerchantProcessorInterface := self merchantProcessorInterface.
	aCreditCard := self validCreditCard.
	
	self should: [aRest checkoutCartId: aRest activeCarts someElement withCreditCardNumber: aCreditCard number withExpirationDate: aCreditCard expirationDate withOwnerName: aCreditCard ownerName.]
		raise: Error 
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'An empty cart cannot be charged.'.
							self assert: aSalesBook = aRest salesBook.
							self assert: aMerchantProcessorInterface numberOfCalls = numberOfCalls]! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 19:01'!
test30CannotCheckoutIfMerchantProcessorRejectedTheOperation
	
	| aRest aCreditCard aMerchantProcessorInterface aSalesBook numberOfCalls someCartId |
	
	aRest := self restWithOneEmptyCart.
	numberOfCalls := self merchantProcessorInterface numberOfCalls.
	aSalesBook := aRest salesBook.
	aMerchantProcessorInterface := self merchantProcessorInterface.
	aCreditCard := self invalidCreditCard.
	
	someCartId := aRest activeCarts someElement.
	aRest addToCart: someCartId aBook: self book1 quantity: 2.
	self should: [aRest checkoutCartId: someCartId withCreditCardNumber: aCreditCard number withExpirationDate: aCreditCard expirationDate withOwnerName: aCreditCard ownerName.]
		raise: Error 
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Could not charge from creditCard'.
							self assert: aSalesBook = aRest salesBook.
							self assert: aMerchantProcessorInterface numberOfCalls = (numberOfCalls + 1)
							]! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 19:05'!
test31CanCheckoutIfMerchantProcessorAllowedTheOperation
	
	| aRest aCreditCard aMerchantProcessorInterface aSalesBook numberOfCalls aSale someCartId |
	
	aRest := self restWithOneEmptyCart.
	numberOfCalls := self merchantProcessorInterface numberOfCalls.
	aSalesBook := aRest salesBook.
	aMerchantProcessorInterface := self merchantProcessorInterface.
	aSale := self aSaleOfThreeBook2.
	aCreditCard := self validCreditCard.
	
	someCartId := aRest activeCarts someElement.
	aRest addToCart: someCartId aBook: self book2 quantity: 3.
	aRest checkoutCartId: someCartId withCreditCardNumber: aCreditCard number withExpirationDate: aCreditCard expirationDate withOwnerName: aCreditCard ownerName.
	
	aSalesBook add: aSale.
	self assert: aMerchantProcessorInterface numberOfCalls = (numberOfCalls + 1).
	self assert: aRest salesBook = aSalesBook.
	
						! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 19:15'!
test32CannotListPurchasesOfAnInvalidUser
	
	| aRest aCreditCard |
	 
	aRest := self restWithOneEmptyCart.
	self should: [aRest listPurchasesForAUser: self invalidUser  withPassword: self password1]
		raise: Error 
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Cannot list puchases of an invalid user.']! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 19:16'!
test33CannotListPurchasesWithInvalidPassword
	
	| aRest aCreditCard |
	 
	aRest := self restWithOneEmptyCart.
	self should: [aRest listPurchasesForAUser: self user1   withPassword: 'xxx']
		raise: Error 
		withExceptionDo: [:anException | 
							self assert: anException messageText = 'Cannot list puchases of an user with an invalid password.']! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 19:57'!
test34CanListPurchasesFromUserWithNoPurchases
	
	| aRest aListOfPurchases |
	 
	aRest := self restWithOneEmptyCart.
	aListOfPurchases := aRest listPurchasesForAUser: self user1 withPassword: self password1.
	self assert: (aListOfPurchases size) = 1.
	self assert: (aListOfPurchases at: 'totalAmount') = 0.! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 20:08'!
test35CanListPurchasesFromUserWithPurchases
	
	| aRest aCreditCard someCartId aListOfPurchases |
	 
	aRest := self restWithOneEmptyCart.
	someCartId := aRest activeCarts someElement.
	aCreditCard := self validCreditCard.	
	someCartId := aRest activeCarts someElement.
	
	aRest addToCart: someCartId aBook: self book1 quantity: 2.
	aRest checkoutCartId: someCartId withCreditCardNumber: aCreditCard number withExpirationDate: aCreditCard expirationDate withOwnerName: aCreditCard ownerName.
	aRest addToCart: someCartId aBook: self book2 quantity: 4.
	aRest addToCart: someCartId aBook: self book1 quantity: 1.
	aRest checkoutCartId: someCartId withCreditCardNumber: aCreditCard number withExpirationDate: aCreditCard expirationDate withOwnerName: aCreditCard ownerName.
		
	aListOfPurchases := aRest listPurchasesForAUser: self user1 withPassword: self password1.
		
	self assert: (aListOfPurchases size) = 3.
	self assert: (aListOfPurchases at: self book1) = 3.
	self assert: (aListOfPurchases at: self book2) = 4.
	self assert: (aListOfPurchases at: 'totalAmount') = ((3 * (self catalog at: self book1)) + (4 * (self catalog at: self book2))).	
	! !

!TusLibrosTest methodsFor: '3 - Test Rest' stamp: 'IgalFrid 6/24/2012 16:55'!
testRest: aRest methodAddToCart: cartId aBook: aBook aQuantity: aQuantity withExceptionMessage: aMessage
	
	|  previousCarts previousActiveCarts |
	
	previousActiveCarts := aRest activeCarts. 
	previousCarts := Dictionary new.
	aRest activeCarts do: [:aCartId | previousCarts at: aCartId put: (aRest getCart: aCartId)].
	
	self should: [aRest addToCart: cartId aBook: aBook quantity: aQuantity]
	raise: Error
	withExceptionDo: [:anExcepcion |
						self assert: anExcepcion messageText = aMessage.
						self assert: aRest activeCarts = previousActiveCarts.
						aRest activeCarts do: [:aCartId | self assert: (aRest getCart: aCartId) = (previousCarts at: aCartId)] 
						]! !

!TusLibrosTest methodsFor: 'users' stamp: 'IgalFrid 6/24/2012 15:38'!
user1 
	
	^'user1'! !

!TusLibrosTest methodsFor: 'creditcards' stamp: 'IgalFrid 6/24/2012 20:02'!
validCreditCard 
	
	^CreditCard ownedBy:self user1 identifiedBy: 1111 expiresOn: (MonthOfYear januaryOf: (Date current year + 1))
 ! !
